@@ -10,7 +10,6 @@ import asyncpg
 import collections
 import collections.abc
 import itertools
-import struct
 import sys
 import time
 import traceback
@@ -539,7 +538,7 @@ class Connection(metaclass=ConnectionMeta):
         :param str query:
             The query to copy the results of.
 
-        :param \*args:
+        :param args:
             Query arguments.
 
         :param output:
@@ -1080,7 +1079,7 @@ class Connection(metaclass=ConnectionMeta):
         try:
             if not self.is_closed():
                 await self._protocol.close(timeout)
-        except Exception:
+        except (Exception, asyncio.CancelledError):
             # If we fail to close gracefully, abort the connection.
             self._abort()
             raise
@@ -1186,24 +1185,16 @@ class Connection(metaclass=ConnectionMeta):
             await self._protocol.close_statement(stmt, protocol.NO_TIMEOUT)
 
     async def _cancel(self, waiter):
-        r = w = None
-
         try:
             # Open new connection to the server
-            r, w = await connect_utils._open_connection(
-                loop=self._loop, addr=self._addr, params=self._params)
-
-            # Pack CancelRequest message
-            msg = struct.pack('!llll', 16, 80877102,
-                              self._protocol.backend_pid,
-                              self._protocol.backend_secret)
-
-            w.write(msg)
-            await r.read()  # Wait until EOF
+            await connect_utils._cancel(
+                loop=self._loop, addr=self._addr, params=self._params,
+                backend_pid=self._protocol.backend_pid,
+                backend_secret=self._protocol.backend_secret)
         except ConnectionResetError as ex:
             # On some systems Postgres will reset the connection
             # after processing the cancellation command.
-            if r is None and not waiter.done():
+            if not waiter.done():
                 waiter.set_exception(ex)
         except asyncio.CancelledError:
             # There are two scenarios in which the cancellation
@@ -1213,7 +1204,7 @@ class Connection(metaclass=ConnectionMeta):
             # the CancelledError, and don't want the loop to warn about
             # an unretrieved exception.
             pass
-        except Exception as ex:
+        except (Exception, asyncio.CancelledError) as ex:
             if not waiter.done():
                 waiter.set_exception(ex)
         finally:
@@ -1221,8 +1212,6 @@ class Connection(metaclass=ConnectionMeta):
                 compat.current_asyncio_task(self._loop))
             if not waiter.done():
                 waiter.set_result(None)
-            if w is not None:
-                w.close()
 
     def _cancel_current_command(self, waiter):
         self._cancellations.add(self._loop.create_task(self._cancel(waiter)))
@@ -1792,13 +1781,18 @@ class _StatementCache:
         return (e._statement for e in self._entries.values())
 
     def clear(self):
-        # First, make sure that we cancel all scheduled callbacks.
-        for entry in self._entries.values():
-            self._clear_entry_callback(entry)
+        # Store entries for later.
+        entries = tuple(self._entries.values())
 
         # Clear the entries dict.
         self._entries.clear()
 
+        # Make sure that we cancel all scheduled callbacks
+        # and call on_remove callback for each entry.
+        for entry in entries:
+            self._clear_entry_callback(entry)
+            self._on_remove(entry._statement)
+
     def _set_entry_timeout(self, entry):
         # Clear the existing timeout.
         self._clear_entry_callback(entry)
