@@ -5,13 +5,15 @@
 # the Apache 2.0 License: http://www.apache.org/licenses/LICENSE-2.0
 
 
+import pathlib
 import sys
 import unittest
 
 
 def suite():
     test_loader = unittest.TestLoader()
-    test_suite = test_loader.discover('.', pattern='test_*.py')
+    test_suite = test_loader.discover(str(pathlib.Path(__file__).parent),
+                                      pattern='test_*.py')
     return test_suite
 
 
